# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .otlp_resource import OtlpResource
from .otlp_scope_logs import OtlpScopeLogs


class OtlpResourceLogs(UniversalBaseModel):
    """
    OTLP resource logs container.
    """

    resource: typing.Optional[OtlpResource] = None
    scope_logs: typing_extensions.Annotated[
        typing.Optional[typing.List[OtlpScopeLogs]], FieldMetadata(alias="scopeLogs")
    ] = pydantic.Field(alias="scopeLogs", default=None)

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
