# This file was auto-generated by Fern from our API Definition.

# This file was auto-generated by Fern from our API Definition.

import typing_extensions

from terminaluse.core.serialization import FieldMetadata


class SquareParams(typing_extensions.TypedDict):
    length_measurement: typing_extensions.Annotated[float, FieldMetadata(alias="lengthMeasurement")]
