# This file was auto-generated by Fern from our API Definition.

# This file was auto-generated by Fern from our API Definition.

import typing

from .circle import CircleParams
from .square import SquareParams

UndiscriminatedShapeParams = typing.Union[CircleParams, SquareParams]
