# This file was auto-generated by Fern from our API Definition.

import typing

VersionEventType = typing.Union[
    typing.Literal[
        "VERSION_CREATED",
        "VERSION_ACTIVATED",
        "VERSION_FAILED",
        "VERSION_DRAINING",
        "VERSION_RETIRED",
        "VERSION_ROLLED_BACK_FROM",
        "VERSION_ROLLED_BACK_TO",
        "VERSION_REDEPLOYED",
        "TASKS_MIGRATED",
        "VERSION_CLEANUP_FAILED",
    ],
    typing.Any,
]
