# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class EnvVarValue(UniversalBaseModel):
    """
    Value with is_secret flag for setting environment variables.
    """

    is_secret: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, value is write-only via API (cannot retrieve). If false, value can be read back via API.
    """

    value: str = pydantic.Field()
    """
    The plaintext value to encrypt and store
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
