# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .cross_env_secret_response import CrossEnvSecretResponse


class CrossEnvSecretsListResponse(UniversalBaseModel):
    """
    Cross-environment secrets view.
    """

    all_environments: typing.List[str] = pydantic.Field()
    """
    All environment names for the agent
    """

    variables: typing.List[CrossEnvSecretResponse] = pydantic.Field()
    """
    All secret keys across environments
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
