# This file was auto-generated by Fern from our API Definition.

# This file was auto-generated by Fern from our API Definition.

import typing

Color = typing.Union[typing.Literal["red", "blue"], typing.Any]
