# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .deployed_secret_key import DeployedSecretKey


class DeployedSecretsResponse(UniversalBaseModel):
    """
    Response for deployed secrets (from Version.secrets_snapshot).
    """

    deployed_at: dt.datetime = pydantic.Field()
    """
    When this version was deployed
    """

    secrets: typing.List[DeployedSecretKey] = pydantic.Field()
    """
    List of secret keys (no values)
    """

    state: typing.Optional[str] = pydantic.Field(default=None)
    """
    Always 'deployed' for this response
    """

    version_id: str = pydantic.Field()
    """
    Version ID this snapshot came from
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
