# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .version_status import VersionStatus


class RegisterContainerResponse(UniversalBaseModel):
    """
    Response from container registration.
    """

    agent_api_key: str = pydantic.Field()
    """
    API key for the agent
    """

    agent_id: str = pydantic.Field()
    """
    Agent ID
    """

    agent_name: str = pydantic.Field()
    """
    Agent name
    """

    branch_id: str = pydantic.Field()
    """
    Branch ID
    """

    version_status: VersionStatus = pydantic.Field()
    """
    Version status after registration
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
