# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SyncCompleteResponse(UniversalBaseModel):
    """
    Response model for sync-complete operation.
    """

    files_count: int = pydantic.Field()
    """
    Number of files processed.
    """

    filesystem_id: str = pydantic.Field()
    """
    The filesystem ID.
    """

    status: str = pydantic.Field()
    """
    Result status: 'COMPLETED' or 'ALREADY_PROCESSED'.
    """

    sync_id: str = pydantic.Field()
    """
    The sync operation ID (for idempotency).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
