# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CliRefreshResponse(UniversalBaseModel):
    """
    Response for CLI token refresh.
    """

    expires_in: int = pydantic.Field()
    """
    Token expiration in seconds
    """

    refresh_token: str = pydantic.Field()
    """
    New refresh token (replaces the old one)
    """

    session_jwt: str = pydantic.Field()
    """
    New session JWT (WorkOS access token)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
