# This file was auto-generated by Fern from our API Definition.

import typing

FilesystemStatus = typing.Union[typing.Literal["CREATING", "READY", "SYNCING_UP", "SYNCING_DOWN", "FAILED"], typing.Any]
