# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.version_event_list_response import VersionEventListResponse
from ..types.version_event_type import VersionEventType


class RawVersionEventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def versions_events_list(
        self,
        version_id: str,
        *,
        event_type: typing.Optional[VersionEventType] = None,
        after_sequence_id: typing.Optional[int] = None,
        before_sequence_id: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        descending: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VersionEventListResponse]:
        """
        List lifecycle events for a version. Events are ordered by sequence_id.

        Parameters
        ----------
        version_id : str

        event_type : typing.Optional[VersionEventType]
            Filter by event type

        after_sequence_id : typing.Optional[int]
            Return events after this sequence_id (for ascending pagination)

        before_sequence_id : typing.Optional[int]
            Return events before this sequence_id (for descending pagination)

        limit : typing.Optional[int]
            Maximum events to return

        descending : typing.Optional[bool]
            Order by sequence_id descending (newest first)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VersionEventListResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"versions/{jsonable_encoder(version_id)}/events",
            method="GET",
            params={
                "event_type": event_type,
                "after_sequence_id": after_sequence_id,
                "before_sequence_id": before_sequence_id,
                "limit": limit,
                "descending": descending,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionEventListResponse,
                    parse_obj_as(
                        type_=VersionEventListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawVersionEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def versions_events_list(
        self,
        version_id: str,
        *,
        event_type: typing.Optional[VersionEventType] = None,
        after_sequence_id: typing.Optional[int] = None,
        before_sequence_id: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        descending: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VersionEventListResponse]:
        """
        List lifecycle events for a version. Events are ordered by sequence_id.

        Parameters
        ----------
        version_id : str

        event_type : typing.Optional[VersionEventType]
            Filter by event type

        after_sequence_id : typing.Optional[int]
            Return events after this sequence_id (for ascending pagination)

        before_sequence_id : typing.Optional[int]
            Return events before this sequence_id (for descending pagination)

        limit : typing.Optional[int]
            Maximum events to return

        descending : typing.Optional[bool]
            Order by sequence_id descending (newest first)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VersionEventListResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"versions/{jsonable_encoder(version_id)}/events",
            method="GET",
            params={
                "event_type": event_type,
                "after_sequence_id": after_sequence_id,
                "before_sequence_id": before_sequence_id,
                "limit": limit,
                "descending": descending,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VersionEventListResponse,
                    parse_obj_as(
                        type_=VersionEventListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
