# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .principal_request import PrincipalRequest
from .terminal_use_resource import TerminalUseResource


class TupleRoleRequest(UniversalBaseModel):
    """
    Request to write/delete a role tuple.
    """

    principal: PrincipalRequest
    resource: TerminalUseResource
    role: str = pydantic.Field()
    """
    Role name (viewer, editor, admin, owner)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
