# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .api_key_response import ApiKeyResponse


class ApiKeyCreateResponse(UniversalBaseModel):
    """
    Response model for API key creation.

    Includes the full API key (shown only once).
    """

    api_key: ApiKeyResponse = pydantic.Field()
    """
    The created API key metadata.
    """

    key: str = pydantic.Field()
    """
    The full API key. Store securely - it will NOT be shown again.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
