# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class SourceUrlPart(UniversalBaseModel):
    """
    Source URL reference part.
    """

    source_id: typing_extensions.Annotated[str, FieldMetadata(alias="sourceId")] = pydantic.Field(alias="sourceId")
    """
    Unique source ID
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Title of the source
    """

    url: str = pydantic.Field()
    """
    URL of the source
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
