# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .version_event_response import VersionEventResponse


class VersionEventListResponse(UniversalBaseModel):
    """
    Response model for listing version events.
    """

    events: typing.List[VersionEventResponse] = pydantic.Field()
    """
    List of version events
    """

    has_more: bool = pydantic.Field()
    """
    Whether there are more events after the last one
    """

    total: int = pydantic.Field()
    """
    Total number of events returned
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
