# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ReasoningPart(UniversalBaseModel):
    """
    Reasoning/thinking content part (Claude's extended thinking).
    """

    provider_metadata: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Any]], FieldMetadata(alias="providerMetadata")
    ] = pydantic.Field(alias="providerMetadata", default=None)
    """
    Provider-specific metadata (e.g., Claude's thinking.signature)
    """

    text: str = pydantic.Field()
    """
    The reasoning text content
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
