# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.cli_refresh_response import CliRefreshResponse
from ..types.cli_token_response import CliTokenResponse
from ..types.http_validation_error import HttpValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCliAuthenticationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def refresh_cli_token(
        self, *, refresh_token: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CliRefreshResponse]:
        """
        Refresh an expired session JWT using a refresh token.

        Note: Refresh tokens are single-use. The response contains a new refresh_token
        that must replace the old one for future refresh operations.

        Parameters
        ----------
        refresh_token : str
            Refresh token from previous authentication

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CliRefreshResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/cli/refresh",
            method="POST",
            json={
                "refresh_token": refresh_token,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CliRefreshResponse,
                    parse_obj_as(
                        type_=CliRefreshResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def exchange_cli_token(
        self, *, code: str, code_verifier: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CliTokenResponse]:
        """
        Exchange authorization code for session JWT using PKCE.

        Parameters
        ----------
        code : str
            Authorization code from OAuth callback

        code_verifier : str
            PKCE code verifier

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CliTokenResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth/cli/token",
            method="POST",
            json={
                "code": code,
                "code_verifier": code_verifier,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CliTokenResponse,
                    parse_obj_as(
                        type_=CliTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCliAuthenticationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def refresh_cli_token(
        self, *, refresh_token: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CliRefreshResponse]:
        """
        Refresh an expired session JWT using a refresh token.

        Note: Refresh tokens are single-use. The response contains a new refresh_token
        that must replace the old one for future refresh operations.

        Parameters
        ----------
        refresh_token : str
            Refresh token from previous authentication

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CliRefreshResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/cli/refresh",
            method="POST",
            json={
                "refresh_token": refresh_token,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CliRefreshResponse,
                    parse_obj_as(
                        type_=CliRefreshResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def exchange_cli_token(
        self, *, code: str, code_verifier: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CliTokenResponse]:
        """
        Exchange authorization code for session JWT using PKCE.

        Parameters
        ----------
        code : str
            Authorization code from OAuth callback

        code_verifier : str
            PKCE code verifier

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CliTokenResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth/cli/token",
            method="POST",
            json={
                "code": code,
                "code_verifier": code_verifier,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CliTokenResponse,
                    parse_obj_as(
                        type_=CliTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
