# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.o_auth_initiate_response import OAuthInitiateResponse
from ..types.o_auth_token_response import OAuthTokenResponse


class RawOAuthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def initiate_oauth(
        self,
        *,
        redirect_uri: str,
        code_challenge: typing.Optional[str] = None,
        connection_id: typing.Optional[str] = None,
        organization_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[OAuthInitiateResponse]:
        """
        Generate authorization URL for OAuth authentication.

        Parameters
        ----------
        redirect_uri : str
            URI to redirect after authentication

        code_challenge : typing.Optional[str]
            PKCE code challenge (S256)

        connection_id : typing.Optional[str]
            Optional WorkOS connection ID for SSO

        organization_id : typing.Optional[str]
            Organization ID to scope the authentication to

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[OAuthInitiateResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/oauth/initiate",
            method="GET",
            params={
                "redirect_uri": redirect_uri,
                "code_challenge": code_challenge,
                "connection_id": connection_id,
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OAuthInitiateResponse,
                    parse_obj_as(
                        type_=OAuthInitiateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def exchange_token(
        self, *, code: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[OAuthTokenResponse]:
        """
        Exchange authorization code for access token.

        Parameters
        ----------
        code : str
            Authorization code from OAuth callback

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[OAuthTokenResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/oauth/token",
            method="POST",
            params={
                "code": code,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OAuthTokenResponse,
                    parse_obj_as(
                        type_=OAuthTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawOAuthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def initiate_oauth(
        self,
        *,
        redirect_uri: str,
        code_challenge: typing.Optional[str] = None,
        connection_id: typing.Optional[str] = None,
        organization_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[OAuthInitiateResponse]:
        """
        Generate authorization URL for OAuth authentication.

        Parameters
        ----------
        redirect_uri : str
            URI to redirect after authentication

        code_challenge : typing.Optional[str]
            PKCE code challenge (S256)

        connection_id : typing.Optional[str]
            Optional WorkOS connection ID for SSO

        organization_id : typing.Optional[str]
            Organization ID to scope the authentication to

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[OAuthInitiateResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/oauth/initiate",
            method="GET",
            params={
                "redirect_uri": redirect_uri,
                "code_challenge": code_challenge,
                "connection_id": connection_id,
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OAuthInitiateResponse,
                    parse_obj_as(
                        type_=OAuthInitiateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def exchange_token(
        self, *, code: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[OAuthTokenResponse]:
        """
        Exchange authorization code for access token.

        Parameters
        ----------
        code : str
            Authorization code from OAuth callback

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[OAuthTokenResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/oauth/token",
            method="POST",
            params={
                "code": code,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OAuthTokenResponse,
                    parse_obj_as(
                        type_=OAuthTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
