# This file was auto-generated by Fern from our API Definition.

import typing

TerminalUseResourceType = typing.Union[
    typing.Literal["member", "org", "namespace", "project", "agent", "filesystem", "task"], typing.Any
]
